(function ($) {
	$.widget("lws.lac_select", {
		/** Fully featured Autocomplete Combobox
		 * @param data-class css class list transfered to autocomplete wrapper.
		 * @param data-name input name set to autocomplete wrapper input (in case label is relevant too).
		 * @param data-delay hit key delay before search trigger (let user finish its term before loading).
		 * @param data-placeholder is a input placeholder.
		 * @param data-shared defines if the input shares his source with other elements of the same type
		 * @param data-required defines if the input value is required in the form. Needs formfields.js
		 * @param data-allownew defines if the input allows the input of new values in the dataSource
		 * @param data-showMore defines if the "show more" button can be used for further search on ajax source.
		 * @see lacmodel.js for source and ajax data options
		 *
		 * @param data-mode defines the allowed working modes :
		 *  -	autocomplete : When a research produces at least one result, the text Input is filled with the first value
		 * 	   	The research only works on the results BEGINNING with the researched chain
		 * 	-	research : The research works on any subchain containing the researched chain. No result is selected by default
		 * 	-	select : Works as a standard select control. Allows empty string "" as option key.
		 **/
		options: {
			classe: '',
			name: '',
			placeholder: '',
			shared: false,
			delay: 300,
			maxwidth: '',
			minlength: 1,
			minoptions: 2,
			minsearch: 1,
			showMore: false,
			mode: 'autocomplete',
			required: false,
			allownew: false,
		},

		_create: function () {
			this._setOptions();
			this._createStructure();
			this.currentIndex = -1;
			this.flagAutocomplete = false;
			this.inHouse = false;
			this.initValue = (this.element.val() != undefined) ? [this.element.val()] : undefined;
			// Set Up Callbacks
			this.resLoad = {
				'target': this,
				'fn': this._ajaxLoading
			};
			this.resChange = {
				'target': this,
				'fn': this._resultChanged
			};

			this._manageModel();
			if (this.element.val() == undefined || this.element.val().length == 0) {
				this.resList = $(this.model).lac_model("getSource");
				if (!jQuery.isEmptyObject(this.resList))
					this._setResList(this.resList);
			}
			this._manageEvents();
			var me = this;
			var timer;
			var keyData = [];
			this.container.keyup(function (event, data) {
				keyData[0] = event.key;
				keyData[1] = event.keyCode;
				sentData = [keyData];
				timer && clearTimeout(timer);
				timer = setTimeout(me._bindD(me._manageSearch, me, sentData), me.options.delay);
			});
		},

		_bind: function (fn, me) {
			return function () {
				return fn.apply(me, arguments);
			};
		},

		_bindD: function (fn, me, data) {
			return function () {
				return fn.apply(me, data);
			};
		},

		// Retrieve options from DOM data if needed
		_setOptions: function () {
			if (this.element.data('placeholder') != undefined)
				this.options.placeholder = this.element.data('placeholder');
			if (this.element.data('required') != undefined)
				this.options.required = this.element.data('required');
			if (this.element.data('delay') != undefined)
				this.options.delay = this.element.data('delay');
			if (this.element.data('class') != undefined)
				this.options.classe = this.element.data('class');
			if (this.element.data('name') != undefined)
				this.options.name = this.element.data('name');
			if (this.element.data('maxwidth') != undefined)
				this.options.maxwidth = this.element.data('maxwidth');
			if (this.element.data('shared') != undefined)
				this.options.shared = this.element.data('shared');
			if (this.element.data('mode') != undefined)
				this.options.mode = this.element.data('mode');
			if (this.element.data('allownew') != undefined)
				this.options.allownew = (this.element.data('allownew').trim().length > 0);
		},

		// Destroys the lac_select
		_destroy: function () {
			if (this.container.css('display') != 'none')
				this.element.show();
			this.container.remove();
		},

		// Control Structure
		_createStructure: function () {
			var ignoreConfirm = '';
			if (this.element.hasClass('lws-ignore-confirm'))
				ignoreConfirm = 'lws-ignore-confirm ';
			reqborder = (this.options.required) ? 'lws-reqborder' : '';
			maxwidth = (this.options.maxwidth) ? 'max-width:' + this.options.maxwidth : '';
			this.container = $("<div>", {
				'class': 'lac-select-wrapper ' + this.options.classe,
				'style': maxwidth,
				'tabIndex': '-1',
				'data-uuid': this.uuid,
			}).append($("<div>", {
				'class': 'lac-select-combo ' + reqborder,
			}).append($("<div>", {
				'class': ignoreConfirm + 'lac-select-input',
				'contenteditable': 'true',
				'tabIndex': '0',
				'data-placeholder': this.options.placeholder,
				'name': this.options.name
			})).append($("<div>", {
				'class': 'lac-select-showmore lws-icon lws-icon-show-more'
			})).append($("<div>", {
				'class': 'lac-select-ddbutton lws-icon lws-icon-nav-down'
			}))).append($("<div>", {
				'class': 'lac-select-list',
				'tabIndex': '-1',
				'data-open': false
			})).append($("<div>", {
				'class': 'lac-select-error'
			})).insertAfter(this.element);
			if (this.element.css('display') == 'none')
				this.container.hide();
			this.element.hide();
			this.showMoreButton = this.container.find('.lac-select-showmore');
			this.selectList = this.container.find('.lac-select-list');
			this.textInput = this.container.find('.lac-select-input');
			this.textInput.prop('autocomplete', 'off');
			if (this.options.mode === 'select') {
				this.textInput.css('user-select', 'none');
				this.textInput.prop('contenteditable', 'false');
			}
			if (this.options.required) this.textInput.lws_required();
		},

		// Manage Disabled field
		_manageEvents: function () {
			if (this.element.prop('disabled')) {
				this.textInput.prop('disabled', true);
				this.container.find('.lac-select-ddbutton').addClass('lws-disabled');
				this.container.off();
			} else {
				this.container.find('.lac-select-ddbutton').removeClass('lws-disabled');
				this.container.on('click', '.lac-select-ddbutton', this._bind(this._showHide, this));
				this.container.on('click', '.lac-select-showmore', this._bind(this._showMore, this));
				this.container.on('click', '.lac-select-item', this._bind(this._selectItem, this));
				this.container.on('focusout', this._bind(this._manageFocus, this));
				this.container.on('click', '.lac-select-input', this._bind(this._manageInputFocus, this));
				this.container.on('keydown', this._bind(this._manageKeys, this));
				this.element.on('change', this._bind(this._changeElement, this));
			}
		},

		currentValue: function () {
			return this.element.val();
		},

		currentText: function () {
			return this.textInput.html();
		},

		// manage the dom object the model is attached to
		// can be unique or shared between controls
		_manageModel: function () {
			if (this.options.shared) {
				if ($('#sha-' + this.options.shared).length) {
					this.model = $('#sha-' + this.options.shared);
				} else {
					this.model = $('<input>', {
						'id': 'sha-' + this.options.shared,
						'type': 'hidden'
					}).appendTo($('body'));
				}
			} else {
				this.model = this.element;
			}
			$(this.model).lac_model({
				'mode': this.options.mode,
				'origin': this
			});
			if (this.initValue != undefined && (this.initValue != '' || this.options.mode == 'select')) {
				$(this.model).lac_model("returnLabels", this.initValue, this, this.options.mode);
			}

		},

		// Construction récursive de la liste des résultats
		_recursiveList: function (resList, depth) {
			var resultList = [];
			var me = this;
			$.each(resList, function (key, selItem) {
				if (selItem.group != undefined) {
					retour = me._recursiveList(selItem.group, depth + 1)
					if (retour.length > 0) {
						if (retour[0][0].className != 'lac-select-optgroup') {
							var item = $("<div>", {
								'class': 'lac-select-optgroup',
								'data-value': selItem.value,
								'html': selItem.label,
							});
							resultList.push(item);
						}
						resultList = $.merge(resultList, retour);
					}
				} else {
					me.selectIndex += 1;
					var item = $("<div>", {
						'class': 'lac-select-item lac-item-' + me.selectIndex,
						'data-value': selItem.value,
						'data-label': selItem.label,
						'data-index': me.selectIndex,
					});
					if (selItem.html != undefined && selItem.html != '') {
						item.html(selItem.html);
					} else {
						item.html(selItem.label);
					}
					if (depth > 0) {
						arrow = $("<div>", { 'class': 'lac-item-arrow lws-icon lws-icon-arrow-right' });
						item.prepend(arrow);
					}
					resultList.push(item);
				}
			})
			return resultList;
		},

		// Reset and set of the Result Dropdown List
		_setResList: function (resList) {
			this.selectList.empty();
			this.currentIndex = -1;
			this.selectIndex = -1;
			var retour = this._recursiveList(resList, 0);
			for (var i = 0; i < retour.length; i++) {
				retour[i].appendTo(this.selectList);
			}
			this.container.find('.lac-select-item').removeClass("lac-highlighted");
		},

		// Select item in the result list
		_selectItem: function (event) {
			item = $(event.currentTarget);
			this._changeValue(item.data('value'));
			this.textInput.text($('<span>').html(item.data('label')).text()); // escape html entities
			this.currentIndex = item.data('index');
			this._closeList();
			this.element.trigger('updated');
		},

		// Event triggered on element value change
		_changeElement: function (event) {
			if (this.inHouse === true) {
				this.inHouse = false;
				this.textInput.focus();
			} else {
				if (this.element.val() == '') {
					this.textInput.text('');
					this.element.trigger('updated');
				} else {
					$(this.model).lac_model("returnLabels", [this.element.val()], this);
				}
			}
		},

		// Internal value change
		_changeValue: function (value) {
			this.inHouse = true;
			this.element.val(value);
			this.element.trigger("change");
		},

		// Show or hide the result list when click on ddbutton
		_showHide: function (event, data) {
			if (!this.selectList.data('open')) {
				if (this.options.mode != 'select') {
					this.flagAutocomplete = true;
				}
				$(this.model).lac_model("returnSource");
			} else {
				this._closeList();
				return false;
			}
		},

		// Click on Show More button
		_showMore: function (event, data) {
			$(this.model).lac_model("showMoreResults", this);
		},

		// Open Results List
		_openList: function () {
			if (this.showMore) this.showMoreButton.show();
			this.selectList.data('open', true);
			this.selectList.show();
		},

		// Close Results List
		_closeList: function () {
			this.showMoreButton.hide();
			this.selectList.data('open', false);
			this.selectList.hide();
		},

		// Input focus depending on mode : select/autocomplete
		_manageInputFocus: function (event) {
			if (this.options.mode == 'select') {
				if (!this.selectList.data('open')) {
					$(this.model).lac_model("returnSource");
				} else {
					this._closeList();
				}
			}
		},

		// Fired when the control looses focus
		// With fix to prevent focusout management when click on a sub-element of the control
		// wrapper must have tabIndex set (preferably to -1) for this to work
		_manageFocus: function (event, data) {
			if (!event.originalEvent) return;
			var target = $(event.originalEvent.relatedTarget).closest('.lac-select-wrapper');
			if (target.length > 0 && target.data('uuid') == this.uuid) return;
			this._handleNewOrIncorrectValue();
			this._closeList();
		},

		_handleNewOrIncorrectValue: function () {
			if (this.textInput.text() != '' && !this._isInResList(this.element.val(), this.textInput.text())) {
				if (this.options.allownew != undefined && this.options.allownew == true) {
					var key = this.textInput.text();
					var newVal = []; newVal[key] = key;
					$(this.model).lac_model("addToSource", newVal, this);
					this._changeValue(key);
				} else {
					this.textInput.text('');
					this._showError('Incorrect Value');
					this.element.trigger('updated');
				}
			}
		},

		_isInResList: function (val, label) {
			var sanitizer = $('<span>');
			for (const selItem of this.resList) {
				if (selItem.value == val && sanitizer.html(selItem.label).text() == label) {
					return true;
				} else if (selItem.group != undefined) {
					for (const groupItem of selItem.group) {
						if (groupItem.value == val && sanitizer.html(groupItem.label).text() == label)
							return true;
					}
				}
			}
			return false;
		},

		// String search
		_manageSearch: function (data) {
			if (data[0].length == 1 && this.options.mode != 'select') {
				this.currentIndex = -1;
				$(this.model).lac_model("research", this.textInput.text(), this);
			};
		},

		// Show Error
		_showError: function (msg) {
			this.container.find(".lac-select-error").html(msg).show().delay(1000).fadeOut(500);
		},

		// Special Keys pressed management
		_manageKeys: function (event, data) {
			if (event.key == 'ArrowDown') {
				this.container.find('.lac-item-' + this.currentIndex).removeClass("lac-highlighted");
				this.currentIndex = (this.currentIndex + 1 > this.selectIndex) ? 0 : this.currentIndex + 1;
				var item = this.container.find('.lac-item-' + this.currentIndex);
				item.addClass("lac-highlighted");
				this.textInput.text(item.data('label'));
				this.selectList.scrollTop(Math.floor(item.position().top) + Math.floor(this.selectList.scrollTop()));
				this._changeValue(item.data('value'));
				if (!this.selectList.data('open')) {
					this._openList();
				}
				this.element.trigger('updated');
				return false;
			}
			if (event.key == 'ArrowUp') {
				this.container.find('.lac-item-' + this.currentIndex).removeClass("lac-highlighted");
				this.currentIndex = (this.currentIndex - 1 < 0) ? this.selectIndex : this.currentIndex - 1;
				var item = this.container.find('.lac-item-' + this.currentIndex);
				item.addClass("lac-highlighted");
				this.textInput.text(item.data('label'));
				this.selectList.scrollTop(Math.floor(item.position().top) + Math.floor(this.selectList.scrollTop()));
				this._changeValue(item.data('value'));
				if (!this.selectList.data('open')) {
					this._openList();
				}
				this.element.trigger('updated');
				return false;
			}
			if (event.key == 'Enter' || event.key == "Escape" || event.key == "Tab") {
				this._handleNewOrIncorrectValue();
				this._closeList();
			}
			if (event.key == 'Backspace' || event.key == "Delete" || event.key == "Suppr") {
				if (this.options.mode == 'select') return;
				this.currentIndex = -1;
				this._changeValue('');
				var me = this;
				setTimeout(function () {
					if (me.textInput.text().length == 0) {
						me.resList = $(me.model).lac_model("getSource");
						me._setResList(me.resList);
					} else {
						me.flagAutocomplete = true;
						$(me.model).lac_model("research", me.textInput.text(), me);
					}
				}, 50);
			}

		},

		// Fill Input With Selected option text
		_inputSelectRange: function () {
			if (this.textInput[0].setSelectionRange) {
				var searchString = this.textInput.text();
				this.textInput.focus();
				if (this.options.mode != 'research') {
					var item = this.container.find('.lac-item-' + this.currentIndex);
					this.textInput.text(item.data('label'));
					this.textInput[0].setSelectionRange(searchString.length, this.textInput.text().length);
					this._changeValue(item.data('value'));
					this.element.trigger('updated');
				}
			}
		},

		// Callback function when loading ajaxdata
		_ajaxLoading: function () {
			this.textInput.addClass("lac-loading");
		},

		// Callback from lac_search
		_resultChanged: function (data) {
			this.textInput.removeClass("lac-loading");

			if (data[1] == 'ok') {
				if (!jQuery.isEmptyObject(data[0])) {
					this.resList = data[0];
					this._setResList(this.resList);
					if (!this.flagAutocomplete) {
						this.currentIndex = 0;
						this._inputSelectRange();
						this.container.find('.lac-item-' + this.currentIndex).addClass("lac-highlighted");
					}
					this.flagAutocomplete = false;
					this.showMore = data[2];
					if (!this.showMore) this.showMoreButton.hide();
					this._openList();
				} else {
					//no results
					this._closeList();
					this.selectList.empty();
				}
			} else if (data[1] == 'init') {
				if (!jQuery.isEmptyObject(data[0])) {
					this.resList = data[0];
					this._setResList(this.resList);
					this.currentIndex = 0;
					this.showMore = data[2];
					if (!this.showMore) this.showMoreButton.hide();
					this.textInput.text(this.selectList.find('.lac-item-0').data('label'));
					if (this.options.mode == 'select') {
						this.resList = $(this.model).lac_model("getSource");
						this._setResList(this.resList);
					}
					this.element.trigger('init');
				}
			} else {
				if (!jQuery.isEmptyObject(data[0])) {
					this.resList = data[0];
					this._setResList(this.resList);
					this._showError(data[1]);
				} else {
					this._closeList();
					this.selectList.empty();
					this._showError(data[1]);
				}
			}
		},

		enableControl: function () {
			if (this.element.prop('disabled') == true) {
				this.element.prop('disabled', false);
				this._manageEvents();
			}
		},
		disableControl: function () {
			if (this.element.prop('disabled') == false) {
				this.element.prop('disabled', true);
				this._manageEvents();
			}
		},

		/* Function to reinitialize the dataSource */
		reInitSource: function () {
			if (!this.options.shared) {
				$(this.model).lac_model("reInit", this);
				if (this.initValue != undefined && (this.initValue != '' || this.options.mode == 'select')) {
					$(this.model).lac_model("returnLabels", this.initValue, this, this.options.mode);
				}
			}
		}

	});

})(jQuery)

jQuery(function ($) {
	$(".lac_select").lac_select();
});