(function ($) {
	$.widget("lws.lws_css_editor", {
		// Run through all selectable objects to :
		// - Create appropriate selectors
		// - Fill the masterTable with selectables, selectors and classes
		// - Style the appropriate selectables
		_create: function () {
			this.cssValues = lwsBase64.toObj(this.element.data('cssvalues'));
			this.defaultValues = lwsBase64.toObj(this.element.data('dftvalues'));
			this.menuTable = {};
			this.cssObject = {};
			this.miscValues = {};
			this.cssChain = "";
			this._createCssObject(this.cssValues);
			this._createMaster();
			this._addBackgroundSwitcher();
			this._setCssEditor();
			var centralDiv = this.element.find(".lwss_centraldiv");
			this.popupShadow = $("<div>").addClass("lwss-popup-shadow").appendTo(centralDiv);
			/* Add Events */
			this.element.on("mouseover", ".lwss_menu_selector", this._bind(this._overSelector, this));
			this.element.on("mouseover", ".lwss_selector", this._bind(this._overSelector, this));
			this.element.on("mouseout", ".lwss_menu_selector", this._bind(this._outSelector, this));
			this.element.on("mouseout", ".lwss_selector", this._bind(this._outSelector, this));
			this.element.on("click", ".lwss_menu_selector", this._bind(this._clickSelector, this));
			this.element.on("click", ".lwss_selector", this._bind(this._clickSelector, this));
			this.element.on("click", ".lwss-main-conteneur", this._bind(this._clickEditor, this));
			this.element.on("click", ".lwss-sidemenu", this._bind(this._clickEditor, this));
			this.element.on("click", ".lwss-btvisual", this._bind(this._switchToVisual, this));
			this.element.on("click", ".lwss-btcss", this._bind(this._switchToCss, this));
			this.element.on("click", ".lwss-reset", this._bind(this._resetCss, this));

			this.myStylingPanel = $("<div>").addClass("lwss_styling_panel").prependTo(centralDiv);
			this.myStylingPanel.lws_styling_panel({ stygen: this });
			this._createCssResetList();
			this.makeCssChain();
		},

		_bind: function (fn, me) {
			return function () {
				return fn.apply(me, arguments);
			};
		},

		// Create Master Object
		_createMaster: function () {
			var me = this;
			this.masterObject = {};
			var myMenu = this.element.find(".lwss_sidemenu");
			var elements = this.element.find(".lwss-visual-conteneur");
			elements = elements.find("[data-selector], .lwss_selectable");
			elements.each(function () {
				/* Create the selector and selectable structures */
				var mySelectable = $(this);
				var myClass = '';
				if(mySelectable.data('selector') != undefined) {
					mySelectable.css('position', 'relative');
					myClass = mySelectable.data('selector').trim();
				}
				if( 0 == myClass.length )
					myClass = "." + mySelectable.attr("class").trim().replace(/\s+/g, ".");

				var myStylableObject = {
					Selectable: mySelectable,
					Selector: me._createSelector(mySelectable, myClass),
					Class: myClass,
					Over: "no",
					Active: "no"
				};
				/** Put the structure inside the master Object **/
				/* If this class isn't used yet */
				if (me.masterObject[myClass] == undefined) {
					me.masterObject[myClass] = {};
					me.masterObject[myClass].elements = [];
					// Get the css values
					me.masterObject[myClass].cssValues = (me.cssObject[myClass] != undefined) ? me.cssObject[myClass] : {
						'idle': {}
					};
					if(me.cssObject[myClass] == undefined){me.cssObject[myClass] = me.masterObject[myClass].cssValues}

					// Set the left menu item
					var menuItemHtml = '';
					if( (-1 != myClass.search("lwss_modify")) || ('text' == mySelectable.data('editable')) ){
						menuItemHtml += "<div class='lwss_menu_selector_icon lws-icon lws-icon-edit'></div>";
					}
					menuItemHtml += "<div class='lwss_menu_selector_text'>" + mySelectable.data('type') + "</div>";
					me.masterObject[myClass].menuItem = $("<div>").addClass("lwss_menu_selector").data('theClass', myClass).html(menuItemHtml).appendTo(myMenu);
				}
				/* Another Element for the class */
				me.masterObject[myClass].elements.push(myStylableObject);
			});
		},

		_createCssResetList: function(){
			var me = this;
			this.cssResetList = {};
			this.myStylingPanel.find('.lwss_value').each(function(){
				me.cssResetList[$(this).data("css")] = '';
			})
		},

		// Set up the CSS editor
		_setCssEditor: function () {
			this.cssConteneur = this.element.find(".lwss-css-conteneur");
			this.cssConteneur.find(".lwss-css-editor")
				.append($('<textarea>', {
					'class': 'lwss-css-textarea',
					'id': 'lwss-cssta-' + this.uuid
				}));
		},

		// Switch to Visual Editor
		_switchToVisual: function (event, data) {
			var visButton = this.element.find('.lwss-btvisual');
			if (visButton.hasClass('lws-active')) {
				return;
			} else {
				this.myStylingPanel.lws_styling_panel("resetActiveValues");
				this.element.find('.lwss-btcss').removeClass('lws-active');
				visButton.addClass('lws-active');
				this.element.find('.lwss-visual-conteneur').css('display', 'flex');
				this.element.find('.lwss-css-conteneur').css('display', 'none');
				this.cssEditor.codemirror.off();
			}
		},

		//Switch to CSS Editor
		_switchToCss: function (event, data) {
			var Button = this.element.find('.lwss-btcss');
			if (Button.hasClass('lws-active')) {
				return;
			} else {
				this.element.find('.lwss-btvisual').removeClass('lws-active');
				Button.addClass('lws-active');
				this.element.find('.lwss-css-conteneur').css('display', 'flex');
				this.element.find('.lwss-visual-conteneur').css('display', 'none');
				if (!this.cssEditor) {
					this.cssEditor = wp.codeEditor.initialize(this.cssConteneur.find('#lwss-cssta-' + this.uuid), cm_settings);
				}
				this.makeCssChain()
				this.cssEditor.codemirror.setValue(this.cssChain);
				var me = this; var typingTimer;
				this.cssEditor.codemirror.on('keyup', function () {
					clearTimeout(typingTimer);
					typingTimer = setTimeout(me._bind(me._editorCssChanged, me), "1000");
				});
				this.cssEditor.codemirror.on('keydown', function () {
					clearTimeout(typingTimer);
				});
			}
		},

		// MouseOver
		_overSelector: function (event, data) {
			var theClass = $(event.currentTarget).data("theClass")
			var current = this.masterObject[theClass];
			current.menuItem.addClass('lwss_over_msel');
			for (let i = 0; i < current.elements.length; i++) {
				current.elements[i].Selector.addClass('lwss_over_sel');
			}
		},

		// MouseOver
		_outSelector: function (event, data) {
			var theClass = $(event.currentTarget).data("theClass")
			var current = this.masterObject[theClass];
			current.menuItem.removeClass('lwss_over_msel');
			for (let i = 0; i < current.elements.length; i++) {
				current.elements[i].Selector.removeClass('lwss_over_sel');
			}
		},

		// Create a graphic Selector for each stylable object
		_createSelector: function (aSelectable, theClass) {
			if (aSelectable.prop('nodeName') == "INPUT" || (aSelectable.prop('nodeName') == "TEXTAREA")) {
				//inputs don't have children, we must trick it
				if (aSelectable.attr('type') == "password") {
					aSelectable.prop('disabled', true);
				}
				var parent = aSelectable.parent();
				var wrapper = $("<div>", {
					"class": "lwss_wrapper"
				}).appendTo(parent);
				aSelectable.detach().appendTo(wrapper)
				theSelector = $("<div>", {
					"class": "lwss_selector lwss_inp_sel",
				}).data('theClass', theClass).appendTo(wrapper);
				//theSelector.outerWidth(aSelectable.outerWidth(true) + 4).outerHeight(aSelectable.outerHeight(true) + 4);
				//theSelector.css('top', aSelectable.position().top - 2).css('left', aSelectable.position().left - 2);
			} else {
				theSelector = $("<div>", {
					"class": "lwss_selector",
				}).data('theClass', theClass).prependTo(aSelectable);
			}
			return $(theSelector);
		},

		_addBackgroundSwitcher: function () {
			var main = this.element.find(".lwss-main-conteneur");
			var bgSwitcher = $("<div>").addClass("lws-bg-switcher").appendTo(main);
			var whiteSwitcher = $("<div>").addClass("lws-white-switcher").appendTo(bgSwitcher);
			whiteSwitcher.click(function () {
				main.css("background", "#ffffff");
			})
			var blackSwitcher = $("<div>").addClass("lws-black-switcher").appendTo(bgSwitcher);
			blackSwitcher.click(function () {
				main.css("background", "#000000");
			})
		},

		// replace selectors on the canvas
		placeSelectors: function () {
			this.element.find('.lwss_inp_sel').each(function () {
				var theSelector = $(this);
				var theSelectable = theSelector.parent().find('.lwss_selectable');
				theSelector.outerWidth(theSelectable.outerWidth(true) + 4).outerHeight(theSelectable.outerHeight(true) + 4);
				theSelector.css('top', theSelectable.position().top - 2).css('left', theSelectable.position().left - 2);
			});
		},

		// Click on a selector (menu or element)
		_clickSelector: function (event) {
			this.element.find('.lwss_selector').removeClass('lwss_sel_sel');
			this.element.find('.lwss_menu_selector').removeClass('lwss_sel_msel');
			var theClass = $(event.currentTarget).data("theClass")
			var current = this.masterObject[theClass];
			current.menuItem.addClass('lwss_sel_msel');
			for (let i = 0; i < current.elements.length; i++) {
				current.elements[i].Selector.addClass('lwss_sel_sel');
			}
			var currentItem = this.masterObject[theClass].elements[0].Selectable;
			var inputId = (currentItem.data('id') != '') ? currentItem.data('id') : '';
			this.element.find(".lwss_info").hide();
			this.myStylingPanel.lws_styling_panel("openStylingPanel", theClass, inputId, currentItem);
			this.myStylingPanel.show();
			event.preventDefault();
			return false;
		},

		// Click outside
		_clickEditor: function (event) {
			var element = ($(event.target).hasClass('lwss_menu_selector_text') || $(event.target).hasClass('lwss_menu_selector_text')) ? $(event.target.parentElement) : $(event.target);
			if (!element.hasClass('lwss_selector') && !element.hasClass('lwss_menu_selector')) {
				this.element.find('.lwss_selector').removeClass('lwss_sel_sel');
				this.element.find('.lwss_menu_selector').removeClass('lwss_sel_msel');
				this.myStylingPanel.lws_styling_panel("setEvent", 'idle');
				this.element.find(".lwss_styling_panel").hide();
				this.element.find(".lwss_info").show();
			}
		},

		// update Master Object if css changed outside
		_updateMaster: function () {
			var me = this;
			$.each(this.cssObject, function (className, element) {
				$.each(element, function (statekey, statevalues) {
					if (me.masterObject[className] != undefined) {
						me.masterObject[className].cssValues[statekey] = statevalues;
					}
				})
			})
		},

		//Reset Css style to default values
		_resetCss: function () {
			this.cssObject = {};
			this._createCssObject(this.defaultValues);
			this._updateMaster();
			this._applyCssObject();
			this.makeCssChain();
			if (this.cssEditor != undefined) this.cssEditor.codemirror.setValue(this.cssChain);
		},

		// css modified in the code editor
		_editorCssChanged: function () {
			var nbErrors = this.cssConteneur.find('.CodeMirror-lint-marker-error').length;
			if (nbErrors == 0 && this.cssEditor != undefined) {
				this._cleanStyleObjects();
				/* No Errors - update master - apply style - save chain */
				var cssContent = this.cssEditor.codemirror.getValue();
				var wip = cssContent.replace(/(?:\r\n|\r|\n|\t)/g, '');
				wip = wip.split('}');
				for (var i = 0; i < wip.length; i++) {
					if (wip[i] != '') {
						var line = wip[i].split('{'); var lselector = line[0].split(':');
						var myClass = lselector[0].trim();
						var stateClass = (lselector[1] == undefined) ? 'idle' : lselector[1].trim();
						var cssValTable = line[1].split(';');
						var cssValues = {}
						for (var j = 0; j < cssValTable.length; j++) {
							if(cssValTable[j].indexOf(':')!=-1){
								cssValues[cssValTable[j].split(':')[0].trim()] = cssValTable[j].split(':')[1].trim();
							}
						}
						/* Update The Main Object : this.cssObject */
						this.cssObject[myClass] = {};
						this.cssObject[myClass][stateClass] = cssValues;
					}
				}
				this._updateMaster();
				this._applyCssObject();
				this.makeCssChain();
			};
		},

		// apply style from css Object
		_applyCssObject: function() {
			$.each(this.cssObject, function (className, element) {
				$.each(element, function (statekey, statevalues) {
					lwssCanvas.find(className).css(statevalues);
				})
			});
		},

		// clean Master Object css values
		_cleanStyleObjects: function () {
			$.each(this.masterObject, function (className, element) {
				element.cssValues = {};
			})
			this.cssObject = {};
		},
		// Create the object which will store the style values
		_createCssObject: function (sourceValues) {
			var me = this;
			lwssCanvas = this.element;
			$.each(sourceValues, function (index, element) {
				element.Selector = element.Selector.replaceAll('::', ':'); // support for ::after and ::before
				var mainClass = (element.Selector.split(":"))[0];
				var event = ((element.Selector.split(",")[0].split(":"))[1] != undefined) ? (element.Selector.split(",")[0].split(":"))[1] : "idle";
				var myClass = (me.cssObject[mainClass] != undefined) ? me.cssObject[mainClass] : {};
				myClass[event] = element.Defaults;
				if (element.Values != null && element.Values != undefined && !Array.isArray(element.Values)) {
					myClass[event] = element.Values;
				}
				me.cssObject[mainClass] = myClass;

				lwssCanvas.find(element.Selector).css(element.Defaults);
				lwssCanvas.find(element.Selector).css(element.Values);
			});
		},

		/* Get the formatted CSS for the CSS Editor and   */
		/* Make the base 64 css Chain for database update */
		makeCssChain: function () {
			lwssCanvas = this.element;
			this.cssChain = "";
			var me = this;
			var theInput = this.element.find(".lwss_editor_chain");
			var pseudoElements = [
				'after','backdrop','before','cue','first-letter','first-line',
				'grammar-error','marker','part','placeholder','selection','slotted','spelling-error'
			];

			$.each(this.cssObject, function (className, element) {
				$.each(element, function (statekey, statevalues) {
					if (statevalues != '') {
						if (statekey == "idle") {
							/* We Apply idle values to the canvas everytime the chain is changed */
							//lwssCanvas.find(className).css(me.cssResetList).css(statevalues);
							me.cssChain += className + "{"
						} else if (statekey == "hover") {
							me.cssChain += className + ":hover, " + className + ".hovered{"
						} else if (statekey == "focus") {
							me.cssChain += className + ":focus, " + className + ".focused{"
						} else {
							// other pseudo~ like before, after, first-child and so on
							me.cssChain += className + ":";
							if( pseudoElements.includes(statekey) )
								me.cssChain += ":"; // pseudo-element needs a second :
							me.cssChain += statekey + "{";
						}
						$.each(statevalues, function (csskey, cssvalue) {
							if (cssvalue != undefined && cssvalue != '') me.cssChain += csskey + ":" + cssvalue + ";";
						});
						me.cssChain += "}\n";
					}
				})
			});
			theInput.val(lwsBase64.encode(this.cssChain));
		},


	});

})(jQuery)


jQuery(function ($) {
	//Main
	$(".lwss_editor").lws_css_editor();
	$(".lwss_editor").click(function (e) {
		if ($(e.target).closest(".lwss-color-selector-relative").length == 0 && $(e.target).closest(".lwss-fontselect-popup").length == 0)
			$(".lwss-popup-shadow").hide();
	});
});